/**
 * The Player control interface will be saved in gPlayerControlInterface
 */
var gPlayerControlInterface;

var gConsole;

/**
 * Method called when the player control interface is available.
 */
function onControlInterfaceAvailable(iface) {
    gConsole.log("Player is ready");

    gPlayerControlInterface = iface.player;

    // Add event listener to retrieve player events
    window.addEventListener("idal-player-event", onPlayerEvent);

    // Initialize status
    sendQueryAndUpdateElement("getMute", "mute");
    sendQueryAndUpdateElement("getFolderName", "directory");
    sendQueryAndUpdateElement("getVolume", "volume");
    sendQueryAndUpdateElement("getFileName", "file");
    sendQueryAndUpdateElement("getVolume", "volume");

    // Query the playback status periodically
    window.setInterval(function() {
        sendQueryAndUpdateElement("getState", "status")
            .then(function(state) {
                if (state !== 'stopped') {
                    sendQueryAndUpdateElement("getPosition", "position");
                    sendQueryAndUpdateElement("getDuration", "duration");
                } else {
                    setElementText("position", 0);
                    setElementText("duration", 0);
                }
            })
    }, 1000);

    // Update the select with the available folders
    runPlayerMethod("getAllFolders")
        .then(function(folders) {
            var group = document.getElementById("foldergroup");
            for (var i = 0 ; i < folders.length ; i++) {
                var option = document.createElement("option");
                option.value = folders[i].number;
                option.innerText = folders[i].name;
                group.appendChild(option);
            }
        });;

    // Update the volume input range
    runPlayerMethod("getVolume")
        .then(function(value) {
            var range = document.getElementById("volume-range");
            range.value = value;
        });
}

/**
 * Callback for player events.
 * It is invoked for each received "idal-player-event".
 */
function onPlayerEvent(e) {
    gConsole.log("Received event: " + JSON.stringify(e.detail, null, 4));
    var detail = e.detail;
    switch (detail.type) {
        case "mute-change":
            setElementText("mute", detail.mute);
            break;

        case "volume-change":
            setElementText("volume", detail.volume);
            break;

        case "folder-enter":
            setElementText("directory", detail.folder.name);
            break;

        case "folder-leave":
            setElementText("directory", "");
            break;

        case "file-start":
            setElementText("file", detail.filename);
            break;

        case "file-end":
            setElementText("file", "");
            break;

        default:
            gConsole.warn("Unhandled event type=" + detail.type);
            break;
    }
}


/**
 * Send a command to the player using the Player control interface.
 *
 * Calling:
 *     runPlayerMethod('setVolume', 4)
 *
 * Will to the same as calling:
 *     gPlayerControlInterface.setVolume(4)
 *
 * The function also display logs in the console (success and errors).
 * It avoid writing lots too many code, but can be quite confusing...
 *
 * @param {string} method is the name of a player method
 * @param {any} args the optional arguments
 */
function runPlayerMethod(method, args) {
    // Convert the arguments to a string, only needed to display them...
    var argz = Array.prototype.slice.call(arguments, 1)
        .map(function(value) {
            return JSON.stringify(value);
        })
        .join(', ');

    return invokePlayerMethod.apply(null, arguments)
        .then(function(result) {
            gConsole.log("Player." + method + "(" + argz + ") => " + JSON.stringify(result, null, 4));
            return result;
        })
        .catch(function(error) {
            var message = "Error: Player." + method + "(" + argz + ") => " +  JSON.stringify(error, null, 4);
            gConsole.error(message);
            throw error;
        });
}

/**
 * Invoke a method on the Player control interface.
 * This function check the availability of the control interface
 * that should be available in the gPlayerControlInterface variable.
 *
 * Calling:
 *     invokePlayerMethod('setVolume', 4)
 *
 * Will to the same as calling:
 *     gPlayerControlInterface.setVolume(4)
 *
 * @param {string} method is the name of a player method
 * @return {Promise} the resulting promise
 */
function invokePlayerMethod(method) {
    if (gPlayerControlInterface !== undefined) {
        if (typeof gPlayerControlInterface[method] === "function") {
            return gPlayerControlInterface[method].apply(null, Array.prototype.slice.call(arguments, 1));
        } else {
            return Promise.reject("No such method: " + method);
        }
    } else {
        return Promise.reject("Control interface not available");
    }
}

/**
 * Send a command to the player and update a given HTMLElement
 * text content with the result of the commmand.
 *
 * @param {String} command is the name of a player method
 * @param {String} elementId is an HTMLElement identifier
 */
function sendQueryAndUpdateElement(command, elementId) {
    return runPlayerMethod(command)
        .then(function(value) {
            //gConsole.log("setElementText: " + elementId +" = " + value);
            setElementText(elementId, value);
            return value;
        });
}

/**
 * Run Player.playFolder() for the selected folder
 */
function playSelectedFolder() {
    var select = document.getElementById("folderselect");
    var value = parseInt(select.options[select.selectedIndex].value);
    runPlayerMethod("playFolder", value);
}

/**
 * @param {string} id is an HTMLElement identifier
 * @param {string} text is the text to set in the HTMLElement
 */
function setElementText(id, text) {
    var element = document.getElementById(id);
    if (element != undefined) {
        element.textContent = text;
    }
}

window.addEventListener("load", function() {
    gConsole = new OnScreenConsole();
    gConsole.initialize("console");

    /**
     * Asynchronously get the control interfaces.
     */
    IDAL.getControlInterfaces()
            .then(onControlInterfaceAvailable);

});

