/**
 * Simple printer application example.
 * 
 * This application creates areas that when clicked print a document.
 * During the printing, all the areas are disabled and are enabled
 * only a short time after the print is over (monkey countermeasure).
 * When the printer is offline, the area are also not enabled.
 * 
 * The buttons definitions are defined in a JSON file containing an array of ButtonSpec.
 */
function SimplePrinterApp() {
    this.printer = null;
    this.jobid = -1;
    this.container = null;
    this.printAllow = false;
    this.afterPrintTimeout = 2000;
    this.isOpen = undefined;
}

SimplePrinterApp.prototype = {
    /**
     * Initialize the application
     * @param {String} url is the URL containing the buttons data (a JSON file)
     */
    initialize: function(url) {
        window.addEventListener("idal-printer-event", this.onPrinterEvent.bind(this));

        this.container = document.getElementById("button-container");

        this.fetchConfiguration(url);
    },

    /**
     * @private
     * @param {String} url is the URL or name (absolute or relative) containing the button definitions
     */
    fetchConfiguration: function(url) {
        var data = null;

        var xhr = new XMLHttpRequest();
        
        var thiz = this;
        xhr.addEventListener("readystatechange", function () {
            if (this.readyState === 4 && this.status == 200) {
                try {
                    var config = JSON.parse(this.responseText);
                    thiz.setConfiguration(config);
                } catch(e) {
                    alert(e.message);
                }
            }
        });
        
        xhr.open("GET", url);
        xhr.setRequestHeader("Accept", "application/json");
        xhr.setRequestHeader("Cache-Control", "no-cache");        
        xhr.send(data);
    },
    
    /**
     * @typedef {Object} ButtonSpec
     * @summary Contains the buttons parameters.
     * @property {String} url is the URL (relative or absolute) of the image to print.
     * @property {String} [class] is the CSS class of the button.
     * @property {Number} [left] is the left position of the button (can be omitted if given in the class).
     * @property {Number} [top] is the top position of the button (can be omitted if given in the class).
     * @property {Number} [width] is the button width (can be omitted if given in the class).
     * @property {Number} [height] is the button height (can be omitted if given in the class).
     */


    setConfiguration: function(config) {
        this.config = config;

        // Time between print
        if (typeof config.printtimeout == "number") {
            this.afterPrintTimeout = config.printtimeout;
        }

        // Set opening time...
        try {
            var opening = this.config.openingtime.split(":");
            this.openingminutes = parseInt(opening[0]) * 60 + parseInt(opening[1]);
        } catch(e) {
            this.openingminutes = 0;
        }

        // Set closing time
        try {    
            var closing = this.config.closingtime.split(":");
            this.closingminutes = parseInt(closing[0]) * 60 + parseInt(closing[1]);
        } catch(e) {
            this.closingminutes = 24 * 60;
        }

        this.createButtons(config.buttons);
        this.onMinuteTick();
    },

    /**
     * @private
     * @param {ButtonSpec[]} config is an array of ButtonSpec
     */
    createButtons: function(buttons) {
        for (var i = 0 ; i < buttons.length ; i++) {
            var button = buttons[i];
            var div = document.createElement("div");

            if (button.class !== undefined) {
                div.className = button.class;
            }
            if (button.left !== undefined) {
                div.style.left = button.left + "px";
            }
            if (button.top !== undefined) {
                div.style.top = button.top + "px";
            }
            if (button.width !== undefined) {
                div.style.width = button.width + "px";
            }
            if (button.height !== undefined) {
                div.style.height = button.height + "px";
            }
            if (button.text !== undefined) {
                div.innerText = button.text;
            }

            div.addEventListener("click", this.getClickDelegate(button));
            this.container.appendChild(div);
        }
    },

    /**
     * Set the printer object
     * @param {Printer} printer 
     */
    setControlInterface: function(iface) {
        this.printer = iface.printer;
        this.logger = iface.logger;

        // Get the printer state to enable printing...
        this.printer.getState()
            .then(function(state) {
                this.setPrintAllowed(state.connected && state.online);
            }.bind(this))
            .catch(function() {});
    },

    /**
     * @private
     * @param {Boolean} allow Is true to allow printing documents.
     * @param {Number} [timeout] The number of milliseconds before executing the action
     */
    setPrintAllowed: function(allow, timeout) {
        var callback = function() {
            if (allow) {
                this.container.classList.remove("disabled");
            } else {
                this.container.classList.add("disabled");
            }
            this.printAllow = allow;
        }.bind(this);

        window.setTimeout(callback, timeout ? timeout : 0);
    },

    /**
     * Convenience method: get an image URL from the button definition
     * @private
     * @param {String} url is an URL or a path relative to the current page.
     */
    getUrl: function(url) {
        if (url.startsWith("http://" || url.startsWith("https://"))) {
            return url;
        }

        return document.URL.substr(0, document.URL.lastIndexOf('/')) + "/" + url;
    },

    /**
     * Convenience method: get a function to execute on a click on a button area.
     * @private
     * @param {Object} button is the button definition
     */
    getClickDelegate: function(button) {
        return function() {
            this.onButtonClick(button);
        }.bind(this);
    },

    /**
     * Callback invoked when the user clicks on an area
     * @private
     * @param {String} url is the URL to print
     */
    onButtonClick: function(button) {
        if (this.printAllow && this.printer) {
            var url = this.getUrl(button.url);
            this.logButtonClick(button);
            this.setPrintAllowed(false);
            this.printer
                .print({ url: url })
                .then(this.onPrintStart.bind(this))
                .catch(this.onPrintError.bind(this));
        }
    },

    logButtonClick: function(button) {
        this.log(button.log ? button.log : "button clicked");
    },

    /**
     * Generic function to log a message
     */
    log: function(message) {
        if (this.logger != null) {
            this.logger.log(this.config.logfile, this.getTimestamp() + " " + message);
        }
    },
    
    getTimestamp: function() {
        var now = new Date();
        var twoDigit = function(n) {
            return ("0" + n).slice(-2)
        }

        return now.getFullYear() + "-" + twoDigit(now.getMonth() + 1) + "-" + twoDigit(now.getDate()) + " " + 
            twoDigit(now.getHours()) + ":" + twoDigit(now.getMinutes()) + ":" + twoDigit(now.getSeconds());
    },

    /**
     * Callback invoked on any print error.
     * @private
     */
    onPrintError: function() {
        // You can display error message here...
        this.log("Print failure");
        this.setPrintAllowed(true, this.afterPrintTimeout);
    },

    /**
     * Callback invoked when a printing has been done.
     * Does not mean the printing is done, can be done, will be done.
     * Does only gives the job identifier...
     * @param {Number} id is the job identifier
     */
    onPrintStart: function(id) {
        // Print has started, watch printer events...
        this.jobid = id;
    },

    /**
     * Callback invoked when a printer related event is received.
     * @param {PrinterStateEvent|JobStateEvent} event is the event
     */
    onPrinterEvent: function(event) {
        var detail = event.detail;
        switch (detail.type) {
            case 'job-state':
                this.onJobStateEvent(detail.job);
                break;

            case 'printer-state':
                this.onPrinterStateEvent(detail.state);
                break;

            default:
                break;
        }
    },

    /**
     * Callback invoked when a print job event is received.
     * @param {JobState} job is the job that has changed.
     */
    onJobStateEvent: function(job) {
        // Only watch the last printed job (our job)...
        if (job.id == this.jobid) {
            switch(job.state) {
                case 'failed':
                    // Sorry for the convenience...
                    this.onPrintError();
                    break;

                case 'printed':
                    // The document has been successfully printer, say thank you...
                    this.setPrintAllowed(true, this.afterPrintTimeout);
                    break;

                case 'finished':
                    // Can print a new one... or wait a few seconds...
                    break;
            }
        }
    },

    /**
     * Callback invoked when a printer event is received.
     * It tells whether the printer is online, the paper is available...
     * @param {PrinterState} state is the printer state
     */
    onPrinterStateEvent: function(state) {
        if (state.online) {
            // Documents can be printed...
            this.setPrintAllowed(true);
        } else {
            // Display other contents or at least don't allow printing
            this.setPrintAllowed(false);
        }
    },

    /**
     * This method is called every minute and is responsible for calling 
     * the onOpeningTime() / onClosingTime() methods.
     */
    onMinuteTick: function() {
        var open = this.isOpenNow();
        if (this.isOpen !== open) {
            this.isOpen = open;
            if (open) {
                this.onOpeningTime();
            } else {
                this.onClosingTime();
            }
        }
        this.setTimeoutTick();
    },

    /**
     * Set a timeout to execute the the opening / close verification every minute.
     * Quite expansive but more reliable when setting the time.
     */
    setTimeoutTick: function() {
        var now = new Date();
        var nextms = 1000 * (61 - (now.getHours() * 3600 + now.getMinutes() * 60 + now.getSeconds()) % 60);
        window.setTimeout(this.onMinuteTick.bind(this), nextms);
    },

    /**
     * Check whether the app in the opening/closing intervale.
     * @return {Boolean} true if the app is open
     */
    isOpenNow: function() {
        var now = new Date();

        // Number of minutes since midnight
        var nowminutes = now.getHours() * 60 + now.getMinutes();

        if ((nowminutes < this.openingminutes) || (nowminutes >= this.closingminutes))
            return false;

        return true;
    },

    onOpeningTime: function() {
        // Do whatever you want to do on opening time (show buttons...)
        this.container.style = "";
    },

    onClosingTime: function() {
        // Do whatever you want to do on closing time (hide buttons, display something else...)
        this.container.style = "display: none;";
    }
};
